// indexed binary heap priority queue
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef BHPQ_H
#define BHPQ_H

/* binary heap priority queue structure */
typedef struct {
	double key1;
	int key2;
} QueueEntry;

typedef struct {
	unsigned int size;
	unsigned int max_size;
	int *handle;
	QueueEntry **data;
} BinaryHeapPriorityQueue;

/* initialise binary heap priority queue */
BinaryHeapPriorityQueue *bhpq_init(const unsigned int size);

/* free binary heap priority queue */
void bhpq_free(BinaryHeapPriorityQueue *bhpq);

/* check if queue empty O(1) */
#define bhpq_empty(bhpq) ((bhpq)->size)

/* get number of entries in queue O(1) */
#define bhpq_size(bhpq) ((bhpq)->size)

/* get total size in bytes of binary heap priority queue */
unsigned int bhpq_storage_size(BinaryHeapPriorityQueue *bhpq);

/* remove entry from head of queue O(log n) */
int bhpq_remove_head(QueueEntry *entry, BinaryHeapPriorityQueue *bhpq);

/* peek at entry at head of queue O(1) */
int bhpq_peek_head(QueueEntry *entry, BinaryHeapPriorityQueue *bhpq);

/* replace entry at head of queue O(log n) */
void bhpq_replace_head(QueueEntry *entry, BinaryHeapPriorityQueue *bhpq);

/* delete entry at head of queue O(log n) */
void bhpq_delete_head(BinaryHeapPriorityQueue *bhpq);

/* insert entry into queue O(log n) */
void bhpq_insert(QueueEntry *entry, BinaryHeapPriorityQueue *bhpq);

#endif
