/*
 * File:   finitelattice.h
 * Author: Francisco J. Romero-Campero
 *
 * Created: January 2010
 */

#ifndef _FINITELATTICE_H
#define	_FINITELATTICE_H

#include <fstream>
#include <iostream>
#include <libxml++/libxml++.h>
#include <map>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>

#include "LatticeBNF.h"
#include "Misc.h"

using namespace std;
using namespace xmlpp;


namespace ECSB
{

class finiteLattice {

public:

    //Constructors
    finiteLattice();
    finiteLattice(const finiteLattice& orig);
    finiteLattice(std::string latticeFile);

    //Default Destructor
    ~finiteLattice();

    //Getters
    std::string getName() { return name; }
    int getDimension() { return dimension; }
    std::string getUnits() { return units; }
    int getScale() { return scale; }
    int getXMin() { return xmin; }
    int getXMax() { return xmax; }
    int getYMin() { return ymin; }
    int getYMax() { return ymax; }
    std::map<std::string,double> getParameters() { return parameters; }
    std::vector< std::vector<std::string> > getBasis() { return basis; }
    std::vector< std::vector<std::string> > getVertices() { return vertices; }
    std::vector< std::vector<std::string> > getNeighbours() {return neighbours;}

    //Setters
    void setName(std::string latticeName) { name = latticeName; }
    void setDimension(int latticeDimension) { dimension = latticeDimension; }
    void setUnits(std::string latticeUnits) { units = latticeUnits; }
    void setScale(int latticeScale) { scale = latticeScale; }
    void setXMin(int latticeXMin) { xmin = latticeXMin; }
    void setXMax(int latticeXMax) { xmax = latticeXMax; }
    void setYMin(int latticeYMin) { ymin = latticeYMin; }
    void setYMax(int latticeYMax) { ymax = latticeYMax; }
    void setParameters(std::map<std::string,double> latticeParameters)
    {
        parameters = latticeParameters;
    }
    void setBasis(std::vector< std::vector<std::string> > latticeBasis)
    {
        basis = latticeBasis;
    }
    void setVertices(std::vector< std::vector<std::string> > latticeVertices)
    {
        vertices = latticeVertices;
    }
    void setNeighbours(std::vector< std::vector<std::string> > latticeNeighbours)
    {
        neighbours = latticeNeighbours;
    }

    //Displayer
    void show();

protected:

    //Auxiliary function to create a lattice from an XML file
    void createFromXML(std::string xmlFile);

private:

  std::string name;
  int dimension;
  std::string units;
  int scale;
  int xmin;
  int xmax;
  int ymin;
  int ymax;
  std::map<std::string,double> parameters;
  std::vector< std::vector<std::string> > basis;
  std::vector< std::vector<std::string> > vertices;
  std::vector< std::vector<std::string> > neighbours;

};

}

#endif	/* _FINITELATTICE_H */

