/*
 * File:   LPPRule.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 18 January 2010, 12:16
 */

#ifndef _LPPRULE_H
#define	_LPPRULE_H

#include <list>
#include <string>
#include <vector>

#include <libxml++/libxml++.h>

using namespace std;
using namespace xmlpp;

namespace ECSB {

class LPPRule {

public:
    //Constructors
    LPPRule();
    LPPRule(const LPPRule& orig);
    LPPRule(xmlpp::Node* ruleSeclaration);

    //Default destructor
    ~LPPRule();

    //Getters
    std::string getName() { return name; }
    std::string getType() { return type; }
    double getConstant() { return constant; }
    std::string getStringConstant() { return stringConstant; }
    std::vector<std::string> getLhsOutside() { return lhsOutside; }
    std::vector<std::string> getLhsInside() { return lhsInside; }
    std::vector<std::string> getLhsTranslocation() { return lhsTranslocation; }
    std::vector<std::string> getRhsOutside() { return rhsOutside; }
    std::vector<std::string> getRhsInside() { return rhsInside; }
    std::vector<std::string> getRhsTranslocation() { return rhsTranslocation; }
    std::vector<int> getVector() { return ruleVector; }
    std::string getLabelLhs() { return labelLhs; }
    std::string getLabel() { return label; }
    std::list<std::string> getObjects();

    //Setters
    void setName(std::string n) { name = n; }
    void setType(std::string t) { type = t; }
    void setConstant(double c) { constant = c; }
    void setStringConstant(std::string s) { stringConstant = s; }
    void setLhsOutside(std::vector<std::string> o) { lhsOutside = o; }
    void setLhsInside(std::vector<std::string> o) { lhsInside = o; }
    void setLhsTranslocation(std::vector<std::string> o) {lhsTranslocation = o;}
    void setRhsOutside(std::vector<std::string> o) { rhsOutside = o; }
    void setRhsInside(std::vector<std::string> o) { rhsInside = o; }
    void setRhsTranslocation(std::vector<std::string> o) {rhsTranslocation=o;}
    void setRuleVector(std::vector<int> v) { ruleVector = v; }
    void setLabelLhs(std::string l) { labelLhs = l; }
    void setLabel(std::string l) { label = l; }

    //Function to instantiate rules defined in a module
    void substituteObjects(std::map<std::string,std::string> substitution);
    void substituteConstants(std::map<std::string,std::string> substitution);
    void substituteConstants(std::map<std::string,double> substitution);
    void substituteLabels(std::map<std::string,std::string> substitution);

    //Displayer
    void show();


protected:
    //Auxiliary function to substitue objects in a vector of objects
    std::vector<std::string> substituteMultiset(
                                map<std::string,std::string> substitution,
                                std::vector<std::string> objects);


private:
    std::string name;
    std::string type;
    double constant;
    std::string stringConstant;
    std::vector<std::string> lhsOutside;
    std::vector<std::string> lhsInside;
    std::vector<std::string> lhsTranslocation;
    std::vector<std::string> rhsOutside;
    std::vector<std::string> rhsInside;
    std::vector<std::string> rhsTranslocation;
    std::vector<int> ruleVector;
    std::string labelLhs;
    std::string label;
};

// function prototypes
//Expand module definitions
std::vector<LPPRule> expand(xmlpp::Node *module);
std::vector<LPPRule> expand(xmlpp::Node *module, std::map<std::string,class ModuleDefinition> partialLibrary);

//Remove duplicate rules
std::vector<LPPRule> removeDuplicates(std::vector<LPPRule>);

//Put individual names to rules
std::vector<LPPRule> individualRuleNames(std::vector<LPPRule> rules, std::string lab);

//Checks if a rule is is a vector of rules
int ruleIn(LPPRule rule, std::vector<LPPRule> rules, int pos);

int ruleEqual(LPPRule r1, LPPRule r2);

}

#endif	/* _LPPRULE_H */

