/*
 * File:   LPPuleset.cpp
 * Author: Francisco J. Romero-Campero
 *
 * Created on 18 January 2010, 11:06
 */

#include <iostream>
#include "LPPRuleSet.h"

using namespace ECSB;
using namespace xmlpp;

//Default constructor
LPPRuleSet::LPPRuleSet()
{
    this->numberRules = 0;
}

//Copy constructor
LPPRuleSet::LPPRuleSet(const LPPRuleSet& orig)
{
    this->label = orig.label;
    this->numberRules = orig.numberRules;
    this->setOfRules = orig.setOfRules;
}

//Default constructor
LPPRuleSet::~LPPRuleSet()
{

}

//Constructor from XML specification
LPPRuleSet::LPPRuleSet(xmlpp::Node* ruleSetDeclaration)
{
    //Get the label of the rule set declaration
    xmlpp::Element* ruleSet = dynamic_cast<xmlpp::Element*>(ruleSetDeclaration);
    this->label = (ruleSet->get_attribute("label"))->get_value();

    //Get all the nodes with the rules declarations
    xmlpp::Node::NodeList rules = ruleSetDeclaration->get_children("rule");

    for(Node::NodeList::iterator r = rules.begin(); r != rules.end(); ++r)
        setOfRules.push_back(LPPRule(*r));

    //Get all the nodes with the modules declarations
    xmlpp::Node::NodeList modules = ruleSetDeclaration->get_children("module");

    for(xmlpp::Node::NodeList::iterator m = modules.begin();
        m != modules.end();
        ++m)
    {
      std::vector<LPPRule> moduleRules = expand(*m);
      this->setOfRules.insert(this->setOfRules.end(),
                              moduleRules.begin(),
                              moduleRules.end());
    }

    //Remove the duplicates coming from different modules
    this->setOfRules = removeDuplicates(this->setOfRules);

    //Rename uniquely each rule
    this->setOfRules = individualRuleNames(this->setOfRules,this->label);

    //Get the number of rules in the rule set
    this->numberRules = this->setOfRules.size();
}

//Getter fucntion to obtain the objects used in the rules
std::list<std::string> LPPRuleSet::getObjects()
{
    std::list<std::string> objects;

    for(std::vector<LPPRule>::iterator r = (this->setOfRules).begin();
        r != (this->setOfRules).end();
        ++r)
    {
        std::list<std::string> ruleObjects = r->getObjects();
        std::list<std::string>::iterator it = objects.begin();
        objects.splice(it,ruleObjects);
    }

    objects.sort();
    objects.unique();

    return objects;
}

void LPPRuleSet::show()
{
    std::cout << "Set of rules with label: " << this->label << std::endl << std::endl;
    std::cout << "Number of rules: " << numberRules << std::endl;
    std::cout << "Rules:" << std::endl;

    for(std::vector<LPPRule>::iterator rule = setOfRules.begin();
        rule != setOfRules.end();
        ++rule)
          (*rule).show();
}
