/*
 * File:   LPPsystem.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 14 January 2010, 14:31
 */

#ifndef _LPPSYSTEM_H
#define	_LPPSYSTEM_H

#include <libxml++/libxml++.h>
#include <map>
#include <string>
#include <vector>

#include "LPPsystemBNF.h"
#include "PSystem.h"
#include "SpatialDistribution.h"
#include "FiniteLattice.h"

using namespace std;
using namespace xmlpp;

namespace ECSB
{

class LPPsystem {

public:

    //Constructors
    LPPsystem();
    LPPsystem(const LPPsystem& orig);
    LPPsystem(std::string fileName);

    //Default destructor
    ~LPPsystem();

    //Getters
    std::string getName() { return name; }
    std::map<std::string,PSystem> getPsystems() {return pSystems; }
    PSystem getPSystem(std::string pSystemName);
    finiteLattice getLattice() { return lattice; }
    spatialDistribution getPositions() { return positions; }
	spatialDistribution* getPositionsPtr() { return &positions; }
	std::vector<std::string> getCombinedAlphabets();
    std::map<std::string,LPPRuleSet> getCombinedRuleSets();
    std::map<std::string,InitialMultiset> getCombinedInitialMultisets();
    unsigned long getNumberOfCompartments();

    //Setters
    void setName(std::string str) { name = str; }
    void setPsystems(std::map<std::string,PSystem> systems){pSystems = systems;}
    void setLattice(finiteLattice lat) { lattice = lat; }
    void setDistribution(spatialDistribution pos) { positions = pos;}

    //Displayer
    void show();

protected:

    //Auxiliary function to create a lattice from an XML file
    void createFromXML(std::string xmlFile);
    //Auxiliary function to add the list of P systems to a LPP system
    void addPSystems(xmlpp::Node::NodeList pSystemsDeclaration);
    
    //Auxiliary functions to translate from our SBML to LPP format
    std::string generateModel(const char* modelFile);
    void generateLibrary(const char* modelFile);
    std::map<std::string,std::vector<ECSB::LPPRule> > obtainModules(const char* modelFile);
    std::map<int,std::string> modulesName(const char* modelFile);
    std::vector<ECSB::LPPRule> obtainRules(std::string compId, Model *sbmlModel);
    std::vector<std::string> obtainReactants(Model* sbmlModel, Reaction* reaction);
    std::vector<std::string> obtainProducts(Model* sbmlModel, Reaction* reaction);
    std::string generateLattice(const char* modelFile);
    std::vector<long> obtainBounds(const char* modelFile);
    bool isDegradation(std::string s);
    std::string ruleString(ECSB::LPPRule rule);
    std::map<std::string, std::vector< std::vector<std::string> > > generatePsystems(const char* modelFile);
    std::list<std::string> obtainAlphabet(Compartment* sbmlCompartment,
                                          Model* sbmlModel,
                                          std::string moduleStr);
    std::string getTemplateID(std::string templateID, Model* sbmlModel);
    std::map<std::string,int> obtainInitial(Model* sbmlModel,
                                            std::string moduleStr);

    void writePsystem(std::string fileLibraryName,
                      std::string pSystemName,
                      std::list<std::string> alphabet,
                      std::map<std::string,int> initials,
                      std::vector<ECSB::LPPRule> rules,
                      std::string modulesID,
                      std::map<int,std::string> moduleNames);

    std::string generateLPPsystem(const char* modelFile,
                           std::string latticeFile,
                           std::map<std::string, std::vector< std::vector<std::string> > > positions);

private:

  std::string name;
  std::map<std::string,PSystem> pSystems;
  finiteLattice lattice;
  spatialDistribution positions;

};

}

#endif	/* _LPPSYSTEM_H */

