/*
 * File:   LPPsystemBNF.cpp
 * Author: Francisco J. Romero-Campero
 *
 * Created on 14 January 2010, 12:27
 */

#include "LPPsystemBNF.h"

using namespace std;
using namespace ECSB;
using namespace xmlpp;
using namespace boost::spirit::classic;

std::string ECSB::generateLPPXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* LPPSystemNode = xmlFile.create_root_node("LPPsystem");

    //Associating LPP system name
    std::string LPPSystemName(tree->value.begin(), tree->value.end());
    LPPSystemName = removeSpaces(LPPSystemName);
    LPPSystemNode->set_attribute("name",LPPSystemName);

    //Adding the SPsystems
    addSPsystems(tree->children.begin(),LPPSystemNode);

    //Adding the lattice
    addLattice(tree->children.begin()+1,LPPSystemNode);

    //Adding the spatial distribution
    addSpatialDistribution(tree->children.begin()+2,LPPSystemNode);

    //Writing the XML file with a formatted style
    std::string fileName = LPPSystemName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;
}

void ECSB::addSPsystems(iter_t SPsystemsDeclaration, xmlpp::Node* LPPSystemNode)
{
    //Adding node containing the list of SP systems
    xmlpp::Element* SPsystemsNode = LPPSystemNode->add_child("listOfPSystems");

    //Adding a node for each SP system including its name and file
    for(iter_t SPsyst = SPsystemsDeclaration->children.begin();
        SPsyst != SPsystemsDeclaration->children.end();
        ++SPsyst)
    {
        //name
        std::string SPsystemName(SPsyst->value.begin(),SPsyst->value.end());
        SPsystemName = removeSpaces(SPsystemName);

        //file
        iter_t fileDeclaration = SPsyst->children.begin();
        std::string SPsystemFile(fileDeclaration->value.begin(),fileDeclaration->value.end());
        SPsystemFile = removeSpaces(SPsystemFile);

        //Add node and attribute
        xmlpp::Element* SPsystemNode = SPsystemsNode->add_child("PSystem");
        SPsystemNode->set_attribute("name",SPsystemName);
        SPsystemNode->set_attribute("file",SPsystemFile);
    }
}


void ECSB::addLattice(iter_t latticeDeclaration, xmlpp::Node* LPPSystemNode)
{
    //Adding node containg the lattice specification: its name and file
    xmlpp::Element* latticeNode = LPPSystemNode->add_child("lattice");

    //Adding attributes to the lattice: name and file
    std::string latticeName(latticeDeclaration->value.begin(),latticeDeclaration->value.end());
    latticeName = removeSpaces(latticeName);
    latticeNode->set_attribute("name",latticeName);

    iter_t latticeFileDef = latticeDeclaration->children.begin();
    std::string latticeFile(latticeFileDef->value.begin(),latticeFileDef->value.end());
    latticeFile = removeSpaces(latticeFile);
    latticeNode->set_attribute("file",latticeFile);

}

void ECSB::addSpatialDistribution(iter_t spatialDistributionDef, xmlpp::Node* LPPSystemNode)
{
    //Adding the list of positions
    xmlpp::Element* positionsNode = LPPSystemNode->add_child("spatialDistribution");

    //Adding each position
    for(iter_t pos = spatialDistributionDef->children.begin();
        pos != spatialDistributionDef->children.end();
        ++pos)
    {
        xmlpp::Element* positionNode = positionsNode->add_child("position");
        std::string SPsystemName(pos->value.begin(),pos->value.end());
        SPsystemName = removeSpaces(SPsystemName);
        positionNode->set_attribute("pSystem",SPsystemName);

        for(iter_t element = pos->children.begin();
            element != pos->children.end();
            ++element)
        {
            std::string elementName(element->value.begin(),element->value.end());
            elementName = removeSpaces(elementName);

            //Adding the parameters used in the positions
            if(elementName == "parameters")
            {
                addParameters(element,positionNode);
            }
            //Adding the coordinates for the positions
            else if(elementName == "coordinates")
            {
                addCoordinates(element,positionNode);
            }
        }

    }

}


void ECSB::addParameters(iter_t parametersDef, xmlpp::Node* positionNode)
{
    for(iter_t para = parametersDef->children.begin();
        para != parametersDef->children.end();
        ++para)
    {
        //Adding individual parameters
        xmlpp::Element* parameterNode = positionNode->add_child("parameter");
        std::string parameterName(para->value.begin(),para->value.end());
        parameterName = removeSpaces(parameterName);
        parameterNode->set_attribute("name",parameterName);

        //Getting lower bound, step and upperbound
        iter_t lowDef = para->children.begin();
        std::string lowStr(lowDef->value.begin(),lowDef->value.end());
        lowStr = removeSpaces(lowStr);
        parameterNode->set_attribute("lowerBound",lowStr);

        iter_t stepDef = para->children.begin() + 1;
        std::string stepStr(stepDef->value.begin(),stepDef->value.end());
        stepStr = removeSpaces(stepStr);
        parameterNode->set_attribute("step",stepStr);

        iter_t upDef = para->children.begin() + 2;
        std::string upStr(upDef->value.begin(),upDef->value.end());
        upStr = removeSpaces(upStr);
        parameterNode->set_attribute("upperBound",upStr);
    }
}


void ECSB::addCoordinates(iter_t coordinatesDef, xmlpp::Node* positionNode)
{
    //Adding x coordinate
    iter_t xDef = (coordinatesDef->children.begin())->children.begin();
    std::string xStr(xDef->value.begin(),xDef->value.end());
    xStr = removeSpaces(xStr);
    xmlpp::Element* xNode = positionNode->add_child("Xcoordinate");
    xNode->set_attribute("value",xStr);

    //Adding y coordinate
    if(coordinatesDef->children.size() == 2)
    {
        iter_t yDef = (coordinatesDef->children.begin() + 1)->children.begin();
        std::string yStr(yDef->value.begin(),yDef->value.end());
        yStr = removeSpaces(yStr);
        xmlpp::Element* yNode = positionNode->add_child("Ycoordinate");
        yNode->set_attribute("value",yStr);

    }

}

