/* 
 * File:   LPPsystemBNF.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 14 January 2010, 16:49
 */

#ifndef _LPPSYSTEMBNF_H
#define	_LPPSYSTEMBNF_H

#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_ast.hpp>
#include <libxml++/libxml++.h>
#include <string>
#include "Misc.h"

using namespace std;
using namespace xmlpp;
using namespace boost::spirit::classic;

namespace ECSB
{

////////////////////////////////////////////////////////////////////////////
//  Our LPP system BNF grammar
////////////////////////////////////////////////////////////////////////////
struct LPPsystem_p: public grammar<LPPsystem_p>
{
    template <typename ScannerT>
    struct definition
    {
        definition(LPPsystem_p const&)
        {

            LPPsystem = discard_node_d[!(comment)] >> discard_node_d[str_p("LPPsystem")] >> root_node_d[LPPsystemName] >>
                            discard_node_d[!(comment)] >>
                            SPsystems >>
                            discard_node_d[!(comment)] >>
                            lattice >>
                            discard_node_d[!(comment)] >>
                            spatialDistribution >>
                        discard_node_d[str_p("endLPPsystem")];

            LPPsystemName = leaf_node_d[ lexeme_d[+(alnum_p)]];

            SPsystems = root_node_d[str_p("SPsystems") ]>>
                            +(SPsystem) >>
                        discard_node_d[str_p("endSPsystems")];

            SPsystem = discard_node_d[!(comment)] >> discard_node_d[str_p("SPsystem")] >> root_node_d[SPsystemName] >> discard_node_d[str_p("from")] >> SPsystemFile;

            SPsystemName = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('_'))]];

            SPsystemFile = leaf_node_d[lexeme_d[+(alnum_p | ch_p('.') | ch_p('_'))]];

            lattice = discard_node_d[str_p("lattice")] >> root_node_d[latticeName] >> discard_node_d[str_p("from")] >> latticeFile;

            latticeName = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('_'))]];

            latticeFile = leaf_node_d[lexeme_d[+(alnum_p | ch_p('_') | ch_p('.'))]];

            spatialDistribution = root_node_d[str_p("spatialDistribution")] >>
                                    +(positions) >>
                                  discard_node_d[str_p("endSpatialDistribution")];

            positions = discard_node_d[!(comment)] >> discard_node_d[str_p("positions")] >> discard_node_d[str_p("for")] >> root_node_d[SPsystemName] >>
                            discard_node_d[!(comment)] >>
                            !(parameters) >>
                            discard_node_d[!(comment)] >>
                            coordinates >>
                        discard_node_d[str_p("endPositions")];

            parameters = root_node_d[str_p("parameters")] >>
                            +(parameter) >>
                         discard_node_d[str_p("endParameters")];

            parameter = discard_node_d[!(comment)] >> discard_node_d[str_p("parameter")] >> root_node_d[parameterName] >>
                        discard_node_d[str_p("=")] >> int_p >> discard_node_d[str_p(":")] >> int_p >> discard_node_d[str_p(":")] >> int_p;

            parameterName = leaf_node_d[ lexeme_d[+(alnum_p)]];

            coordinates = root_node_d[str_p("coordinates")] >>
                            discard_node_d[!(comment)] >>
                            xCoordinate >>
                            discard_node_d[!(comment)] >>
                            !(yCoordinate) >>
                          discard_node_d[str_p("endCoordinates")];

            xCoordinate = root_node_d[str_p("x")] >> discard_node_d[str_p("=")] >> leaf_node_d[ lexeme_d[+(alnum_p | ch_p('+') | ch_p('-') | ch_p('*') | ch_p('/'))]];
            yCoordinate = root_node_d[str_p("y")] >> discard_node_d[str_p("=")] >> leaf_node_d[ lexeme_d[+(alnum_p | ch_p('+') | ch_p('-') | ch_p('*') | ch_p('/'))]];

            commentLine = discard_node_d[ch_p('#')] >> discard_node_d[+(alnum_p | ch_p('_') | ch_p('+') | ch_p('.') | ch_p(':') | ch_p(',') | ch_p('-') | ch_p('*') | ch_p('/') | ch_p('(') | ch_p(')'))] >> discard_node_d[ch_p('#')];

            comment = +(commentLine);
        }

        rule<ScannerT> LPPsystem, LPPsystemName, SPsystems, SPsystem, SPsystemName, SPsystemFile, lattice, latticeName, latticeFile, spatialDistribution, positions, parameters, parameter, parameterName, coordinates, xCoordinate, yCoordinate, commentLine, comment;

        rule<ScannerT> const&
        start() const { return LPPsystem; }
    };
};

std::string generateLPPXML(iter_t tree);
void addSPsystems(iter_t SPsystemsDeclaration, xmlpp::Node* LPPSystemNode);
void addLattice(iter_t latticeDeclaration, xmlpp::Node* LPPSystemNode);
void addSpatialDistribution(iter_t spatialDistributionDef, xmlpp::Node* LPPSystemNode);
void addParameters(iter_t parametersDef, xmlpp::Node* positionNode);
void addCoordinates(iter_t coordinatesDef, xmlpp::Node* positionNode);

}

#endif	/* _LPPSYSTEMBNF_H */