/*
 * File:   latticeBNF.cpp
 * Author: Francisco J. Romero-Campero
 *
 * Created on 14 January 2010, 12:27
 */

#include "LatticeBNF.h"

using namespace std;
using namespace ECSB;
using namespace xmlpp;
using namespace boost::spirit::classic;

//Function to generate an XML file for the specification of a lattice from
//a text file in accordance with our BNF grammar
std::string ECSB::generateLatticeXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* latticeNode = xmlFile.create_root_node("lattice");

    //Associated lattice name
    std::string latticeName(tree->value.begin(), tree->value.end());
    latticeName = removeSpaces(latticeName);
    latticeNode->set_attribute("name",latticeName);

    for(iter_t child = tree->children.begin(); child != tree->children.end(); ++child)
    {
        std::string childName(child->value.begin(),child->value.end());
        childName = removeSpaces(childName);

        if(childName == "dimension")
        {
            //Adding the dimension
            iter_t dimDef = child->children.begin();
            std::string dim(dimDef->value.begin(),dimDef->value.end());
				dim = removeSpaces(dim);
            latticeNode->set_attribute("dimension",dim);
        }
        else if(childName == "scale")
	{
            //Adding the scale
            iter_t scaleDef = child->children.begin();
            std::string scale(scaleDef->value.begin(),scaleDef->value.end());
            scale = removeSpaces(scale);
            latticeNode->set_attribute("scale",scale);
	}
        else if(childName == "units")
        {
            //Adding the units
            iter_t unitsDef = child->children.begin();
            std::string units(unitsDef->value.begin(),unitsDef->value.end());
            units = removeSpaces(units);
            latticeNode->set_attribute("units",units);
        }
        else if(childName == "xmin")
        {
            //Adding the units
            iter_t xminDef = child->children.begin();
            std::string xmin(xminDef->value.begin(),xminDef->value.end());
            xmin = removeSpaces(xmin);
            latticeNode->set_attribute("xmin",xmin);
        }
        else if(childName == "xmax")
        {
            //Adding the units
            iter_t xmaxDef = child->children.begin();
            std::string xmax(xmaxDef->value.begin(),xmaxDef->value.end());
            xmax = removeSpaces(xmax);
            latticeNode->set_attribute("xmax",xmax);
        }
        else if(childName == "ymin")
        {
            //Adding the units
            iter_t yminDef = child->children.begin();
            std::string ymin(yminDef->value.begin(),yminDef->value.end());
            ymin = removeSpaces(ymin);
            latticeNode->set_attribute("ymin",ymin);
        }
        else if(childName == "ymax")
        {
            //Adding the units
            iter_t ymaxDef = child->children.begin();
            std::string ymax(ymaxDef->value.begin(),ymaxDef->value.end());
            ymax = removeSpaces(ymax);
            latticeNode->set_attribute("ymax",ymax);
        }
        else if(childName == "parameters")
        {
            for(iter_t param = child->children.begin(); param != child->children.end(); ++param)
            {
                //Adding individual parameter
                std::string parameterName(param->value.begin(),param->value.end());
                parameterName = removeSpaces(parameterName);
                iter_t valueDef = param->children.begin();
                std::string valueStr(valueDef->value.begin(),valueDef->value.end());
                valueStr = removeSpaces(valueStr);
                xmlpp::Element* parameterNode = latticeNode->add_child("parameter");
                parameterNode->set_attribute("name",parameterName);
                parameterNode->set_attribute("value",valueStr);
            }
        }
        else if(childName == "basis")
        {
            xmlpp::Element* basisNode = latticeNode->add_child("basis");
            for(iter_t basis = child->children.begin(); basis != child->children.end(); ++basis)
            {
                //Adding individual vector
                xmlpp::Element* vectorNode = basisNode->add_child("vector");

                iter_t xComp = basis->children.begin();
                iter_t yComp = basis->children.begin() + 1;

                std::string xValue(xComp->value.begin(),xComp->value.end());
                xValue = removeSpaces(xValue);
                std::string yValue(yComp->value.begin(),yComp->value.end());
                yValue = removeSpaces(yValue);

                vectorNode->set_attribute("x",xValue);
                vectorNode->set_attribute("y",yValue);
            }
        }
        else if(childName == "vertices")
        {
            xmlpp::Element* verticesNode = latticeNode->add_child("vertices");
            for(iter_t basis = child->children.begin(); basis != child->children.end(); ++basis)
            {
                //Adding individual vector
                xmlpp::Element* vectorNode = verticesNode->add_child("vector");

                iter_t xComp = basis->children.begin();
                iter_t yComp = basis->children.begin() + 1;

                std::string xValue(xComp->value.begin(),xComp->value.end());
                xValue = removeSpaces(xValue);
                std::string yValue(yComp->value.begin(),yComp->value.end());
                yValue = removeSpaces(yValue);

                vectorNode->set_attribute("x",xValue);
                vectorNode->set_attribute("y",yValue);
            }
        }
        else if(childName == "neighbours")
        {
            xmlpp::Element* neighboursNode = latticeNode->add_child("neighbours");
            for(iter_t basis = child->children.begin(); basis != child->children.end(); ++basis)
            {
                //Adding individual vector
                xmlpp::Element* vectorNode = neighboursNode->add_child("vector");

                iter_t xComp = basis->children.begin();
                iter_t yComp = basis->children.begin() + 1;

                std::string xValue(xComp->value.begin(),xComp->value.end());
                xValue = removeSpaces(xValue);
                std::string yValue(yComp->value.begin(),yComp->value.end());
                yValue = removeSpaces(yValue);

                vectorNode->set_attribute("x",xValue);
                vectorNode->set_attribute("y",yValue);
            }
        }


    }

    //Writing the XML file with a formatted style
    std::string fileName = latticeName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;
}

