/* 
 * File:   latticeBNF.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 14 January 2010, 12:27
 */

#ifndef _LATTICEBNF_H
#define	_LATTICEBNF_H

#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_ast.hpp>
#include <libxml++/libxml++.h>
#include <string>
#include "Misc.h"

using namespace std;
using namespace xmlpp;
using namespace boost::spirit::classic;

namespace ECSB 
{

////////////////////////////////////////////////////////////////////////////
//  Our Library BNF grammar
////////////////////////////////////////////////////////////////////////////
struct lattice_p: public grammar<lattice_p>
{
    template <typename ScannerT>
    struct definition
    {
        definition(lattice_p const&)
        {

            Lattice = discard_node_d[!(comment)] >> discard_node_d[str_p("lattice")] >> root_node_d[latticeName] >>
                        discard_node_d[!(comment)] >> dimension >>
                        !(discard_node_d[!(comment)] >> scale) >>
                        !(discard_node_d[!(comment)] >> units) >>
                        xmin >>
                        xmax >>
                        ymin >>
                        ymax >>
                        !(parameters) >>
                        discard_node_d[!(comment)] >> basis >>
                        !(discard_node_d[!(comment)] >> vertices) >>
                        discard_node_d[!(comment)] >> neighbours >>
                      discard_node_d[str_p("endLattice")];

            latticeName = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('_'))]];

            dimension = root_node_d[str_p("dimension")] >> ( ch_p('1') | ch_p('2') );

            scale = root_node_d[str_p("scale")] >> int_p;

            units = root_node_d[str_p("units")] >> unitName;

            unitName = leaf_node_d[ lexeme_d[+(alnum_p)]];

            xmin = discard_node_d[!(comment)] >> root_node_d[str_p("xmin")] >> int_p;

            xmax = discard_node_d[!(comment)] >> root_node_d[str_p("xmax")] >> int_p;

            ymin = discard_node_d[!(comment)] >> root_node_d[str_p("ymin")] >> int_p;

            ymax = discard_node_d[!(comment)] >> root_node_d[str_p("ymax")] >> int_p;

            parameters = discard_node_d[!(comment)] >> root_node_d[str_p("parameters")] >>
                            +parameter >>
                         discard_node_d[str_p("endParameters")];

            parameter = discard_node_d[!(comment)] >> discard_node_d[str_p("parameter")] >> root_node_d[parameterName]  >>
                    discard_node_d[str_p("value")] >> discard_node_d[ch_p('=')] >>
                    real_p;

            parameterName = leaf_node_d[ lexeme_d[+(alnum_p)]];

            basis = root_node_d[str_p("basis")] >>
                        +vectorDef >>
                    discard_node_d[str_p("endBasis")];

            vectorDef = discard_node_d[ch_p('(')] >> vectorComponent >>
                            root_node_d[ch_p(',')] >>
                        vectorComponent >> discard_node_d[ch_p(')')];

            vectorComponent = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('-') | ch_p('/'))]];

            vertices = root_node_d[str_p("vertices")] >>
                        +vectorDef >>
                       discard_node_d[str_p("endVertices")];

            neighbours = root_node_d[str_p("neighbours")] >>
                            +vectorDef >>
                         discard_node_d[str_p("endNeighbours")];

            commentLine = discard_node_d[ch_p('#')] >> discard_node_d[+(alnum_p | ch_p('_') | ch_p('+') | ch_p('.') | ch_p(':') | ch_p(',') | ch_p('-') | ch_p('*') | ch_p('/') | ch_p('(') | ch_p(')') | ch_p('='))] >> discard_node_d[ch_p('#')];

            comment = +(commentLine);
        }

        rule<ScannerT> Lattice, latticeName, dimension, scale, units, unitName, xmin, xmax, ymin, ymax, parameters, parameter, parameterName, basis, vectorDef, vectorComponent, vertices, neighbours, commentLine, comment;

        rule<ScannerT> const&
        start() const { return Lattice; }
    };
};


//Function to generate the xml file from the text file
std::string generateLatticeXML(iter_t tree);

}

#endif	/* _LATTICEBNF_H */

