/*
 * File:   moduledefinition.cpp
 * Author: fran
 *
 * Created on 19 January 2010, 11:29
 */

#include "Misc.h"
#include "ModuleDefinition.h"
#include "LPPRule.h"

using namespace std;
using namespace xmlpp;
using namespace ECSB;

//Default constructor
ModuleDefinition::ModuleDefinition()
{

}

//Copy constructor
ModuleDefinition::ModuleDefinition(const ModuleDefinition& orig)
{
    this->constants = orig.constants;
    this->labels = orig.labels;
    this->lowerBounds = orig.lowerBounds;
    this->name = orig.name;
    this->objects = orig.objects;
    this->precisions = orig.precisions;
    this->rules = orig.rules;
    this->scales = orig.scales;
    this->upperBounds = orig.upperBounds;
}

//Default destructor
ModuleDefinition::~ModuleDefinition()
{

}

//Constructor from an XML specification
ModuleDefinition::ModuleDefinition(xmlpp::Node* moduleDeclaration,
                                   std::map<std::string,ModuleDefinition>
                                        partialLibrary)
{
    //We down cast the module definition in order to obtain its attributes.
    xmlpp::Element* declaration =
            dynamic_cast<xmlpp::Element*>(moduleDeclaration);

    //Module name
    this->name = (declaration->get_attribute("name"))->get_value();

    //Module objects
    xmlpp::Node::NodeList setOfObjectsList =
            moduleDeclaration->get_children("setOfObjects");

    xmlpp::Node* objectsListDeclaration = setOfObjectsList.front();

    xmlpp::Node::NodeList objectsList =
            objectsListDeclaration->get_children("object");

    for(xmlpp::Node::NodeList::iterator it = objectsList.begin();
        it != objectsList.end();
        ++it)
    {
        xmlpp::Element* objectDeclaration = dynamic_cast<xmlpp::Element*>(*it);
        xmlpp::Attribute* objectName = objectDeclaration->get_attribute("name");
        this->objects.push_back(objectName->get_value());
    }

    //Module constants
    xmlpp::Node::NodeList setOfConstantsList =
            moduleDeclaration->get_children("setOfConstants");

    xmlpp::Node* constantsListDeclaration = setOfConstantsList.front();

    xmlpp::Node::NodeList constantsList =
            constantsListDeclaration->get_children("constant");

    for(xmlpp::Node::NodeList::iterator it = constantsList.begin();
        it != constantsList.end();
        ++it)
    {
        xmlpp::Element* constantDeclaration =
                dynamic_cast<xmlpp::Element*>(*it);
        xmlpp::Attribute* constantName =
                constantDeclaration->get_attribute("name");
        std::string constantNameValue = constantName->get_value();
        this->constants.push_back(constantNameValue);

        xmlpp::Attribute* constantLowerBound =
                constantDeclaration->get_attribute("lowerBound");
        if(constantLowerBound != NULL)
	{
            double constantLowerBoundValue =
                fromStringTo<double>(constantLowerBound->get_value());
            this->lowerBounds[constantNameValue] = constantLowerBoundValue;
	}

        xmlpp::Attribute* constantUpperBound =
                constantDeclaration->get_attribute("upperBound");
        if(constantUpperBound != NULL)
	{
            double constantUpperBoundValue =
                fromStringTo<double>(constantUpperBound->get_value());
            this->upperBounds[constantNameValue] = constantUpperBoundValue;
	}

        xmlpp::Attribute* constantPrecision =
                constantDeclaration->get_attribute("precision");
        if(constantPrecision != NULL)
	{
            double constantPrecisionValue =
                fromStringTo<double>(constantPrecision->get_value());
            this->precisions[constantNameValue] = constantPrecisionValue;
	}

        xmlpp::Attribute* constantScale =
                constantDeclaration->get_attribute("scale");
        if(constantScale != NULL)
	{
            std::string constantScaleValue = constantScale->get_value();
            this->scales[constantNameValue] = constantScaleValue;
	}
    }

    //Module labels
    xmlpp::Node::NodeList setOfLabelsList =
            moduleDeclaration->get_children("setOfLabels");

    xmlpp::Node* labelsListDeclaration = setOfLabelsList.front();
    xmlpp::Node::NodeList labelsList =
            labelsListDeclaration->get_children("label");

    for(xmlpp::Node::NodeList::iterator it = labelsList.begin();
        it != labelsList.end();
        ++it)
    {
        xmlpp::Element* labelDeclaration = dynamic_cast<xmlpp::Element*>(*it);
        xmlpp::Attribute* labelName = labelDeclaration->get_attribute("name");
        this->labels.push_back(labelName->get_value());
    }

    //Module rules
    xmlpp::Node::NodeList setOfRulesList =
            moduleDeclaration->get_children("setOfRules");

    xmlpp::Node* rulesListDeclaration = setOfRulesList.front();

    //Get all the nodes with the rules declarations
    xmlpp::Node::NodeList rulesList =
            rulesListDeclaration->get_children("rule");

    for(xmlpp::Node::NodeList::iterator it = rulesList.begin();
        it != rulesList.end();
        ++it)
    {
      rules.push_back(LPPRule(*it));
    }

    //Get all the nodes with the modules declarations
    xmlpp::Node::NodeList moduleList =
            rulesListDeclaration->get_children("module");

    for(xmlpp::Node::NodeList::iterator m = moduleList.begin();
        m != moduleList.end();
        ++m)
    {
        std::vector<LPPRule> moduleRules = expand(*m, partialLibrary);
        rules.insert(rules.end(),moduleRules.begin(),moduleRules.end());
    }

}

//Instantiation a module definition with specific substitution maps
std::vector<LPPRule> ModuleDefinition::instantiate(std::map<std::string,std::string>
                                                        objectsSubstitution,
                                                   std::map<std::string,string>
                                                        constantsSubstitution,
                                                   std::map<std::string,std::string>
                                                        labelsSubstitution)
{
    std::vector<LPPRule> instantiatedRules = this->rules;

    for(std::vector<LPPRule>::iterator r = instantiatedRules.begin();
      r != instantiatedRules.end();
      ++r)
    {
        r->substituteObjects(objectsSubstitution);
        r->substituteConstants(constantsSubstitution);
        r->substituteLabels(labelsSubstitution);
    }

    return instantiatedRules;
}




//Displayer
void ModuleDefinition::show()
{

  std::cout << "---------------------------------------" << std::endl;
  std::cout << " Module name: " << this->name << std::endl << std::endl;
  std::cout << " \tObjects: ";

  for(vector<string>::iterator it = this->objects.begin();
      it != this->objects.end();
      ++it)
  {
      std::cout << *it << " ";
  }

  std::cout << endl;

  std::cout << "\tConstants: ";

  for(std::vector<std::string>::iterator it = this->constants.begin();
      it != this->constants.end();
      ++it)
    {
      std::cout << *it << "=" << lowerBounds[*it] << ":" << precisions[*it];
      std::cout << ":" << upperBounds[*it] << " " << "scale:" << scales[*it];
    }

  std::cout << std::endl;

  std::cout << "\tLabels: ";

  for(std::vector<std::string>::iterator it = this->labels.begin();
      it != this->labels.end();
      ++it)
    {
      std::cout << *it << " ";
    }

  std::cout << std::endl;

  std::cout << "\tRules: " << std::endl;

  //std::vector<LPPRule> all_rules;

  for(std::vector<LPPRule>::iterator it = this->rules.begin();
      it != this->rules.end();
      ++it)
    {
      std::cout << "\t\t";
      it->show();
    }

  std::cout << "---------------------------------------" << std::endl;

}

ModuleDefinition ECSB::renameVariables(ModuleDefinition module)
{
    std::vector<std::string> renameObjects;
    std::vector<std::string> objects = module.getObjects();

    for(std::vector<std::string>::iterator o = objects.begin();
        o != objects.end();
        ++o)
    {
        std::string renameObject = *o + "'";
        renameObjects.push_back(renameObject);
    }

    module.setObjects(renameObjects);

    std::map<std::string,std::string> objectSubstitution = createSubstitution(objects,renameObjects);
    std::map<std::string,std::string> constantSubstitution = createSubstitution(module.getConstants(),module.getConstants());
    std::map<std::string,std::string> labelSubstitution = createSubstitution(module.getLabels(),module.getLabels());

    std::vector<LPPRule> renameRules = module.instantiate(objectSubstitution, constantSubstitution, labelSubstitution);

    module.setRules(renameRules);

    return module;

}
