/*
 * File:   ModuleDefinition.h
 * Author: fran
 *
 * Created on 19 January 2010, 11:29
 */

#ifndef _MODULEDEFINITION_H
#define	_MODULEDEFINITION_H

#include <map>
#include <string>
#include <vector>

#include <libxml++-2.6/libxml++/libxml++.h>

#include "LPPRule.h"

using namespace std;
using namespace xmlpp;

namespace ECSB {

class ModuleDefinition {

public:
    //Constructors
    ModuleDefinition();
    ModuleDefinition(const ModuleDefinition& orig);
    ModuleDefinition(xmlpp::Node* moduleDeclaration,
                     std::map<std::string,ModuleDefinition> partialLibrary);

    //Destructor
    ~ModuleDefinition();

    //Getters
    std::string getName() { return name; }
    std::vector<std::string> getObjects() { return objects; }
    std::vector<std::string> getConstants() { return constants; }
    std::map<std::string,double> getUpperBounds() { return upperBounds; }
    std::map<std::string,double> getLowerBounds() { return lowerBounds; }
    std::map<std::string,double> getPrecisions() { return precisions; }
    std::map<std::string,std::string> getScales() { return scales; }
    std::vector<std::string> getLabels() { return labels; }
    std::vector<LPPRule> getRules() { return rules; }

    //Setters
    void setName(std::string n) { name = n; }
    void setObjects(std::vector<std::string> o) { objects = o; }
    void setConstants(std::vector<std::string> c) { constants = c; }
    void setUpperBounds(std::map<std::string,double> ub) { upperBounds = ub; }
    void setLowerBounds(std::map<std::string,double> lb) { lowerBounds = lb; }
    void setPrecisions(std::map<std::string,double> p) { precisions = p; }
    void setScales(std::map<std::string,std::string> s) { scales = s; }
    void setLabels(std::vector<std::string> l) { labels = l; }
    void setRules(std::vector<LPPRule> r) { rules = r; }

    //Displayer
    void show();

    //Function to instantiate a module definition with specific substitution
    //maps
    std::vector<LPPRule> instantiate(std::map<std::string,std::string>
                                            objectsSubstitution,
                                     std::map<std::string,string>
                                            constantsSubstitution,
				     std::map<std::string,std::string>
                                            labelsSubstitution);


private:
    std::string name;
    std::vector<std::string> objects;
    std::vector<std::string> constants;
    std::map<std::string,double> upperBounds;
    std::map<std::string,double> lowerBounds;
    std::map<std::string,double> precisions;
    std::map<std::string,std::string> scales;
    std::vector<std::string> labels;
    std::vector<LPPRule> rules;

};

// function prototypes
ModuleDefinition renameVariables(ModuleDefinition module);

}

#endif	/* _MODULEDEFINITION_H */

