/* 
 * File:   ModuleLibrary.cpp
 * Author: Francisco J. Romero-Campero
 * 
 * Created on 18 January 2010, 15:14
 */

#include "ModuleLibraryBNF.h"
#include "ModuleLibrary.h"

using namespace std;
using namespace ECSB;
using namespace xmlpp;

//Default constructor
ModuleLibrary::ModuleLibrary()
{

}

//Copy constructor
ModuleLibrary::ModuleLibrary(const ModuleLibrary& orig)
{
    this->modules = orig.modules;
    this->name = orig.name;
}

//Default destructor
ModuleLibrary::~ModuleLibrary()
{

}

//Constructor from a file
ModuleLibrary::ModuleLibrary(std::string fileName)
{
    //Varibale for the xml file with the definition of a lattice
    std::string xmlFile;

    //Getting the extesion of the file
    size_t pos = fileName.find('.');
    std::string fileExtension = fileName.substr(pos+1);

    if(fileExtension == "plb")
    {        
        //The module library parser generated from our BNF grammar
        ModuleLibrary_p moduleLibraryParser;

        //Generate a string with the content of the file
        std::string stringFile = fileToString(fileName);

        //We parse the content of the file to generate an abstract syntax tree
        tree_parse_info<> result = ast_parse(stringFile.c_str(),
                                             moduleLibraryParser,
                                             space_p);

        if (result.full) //When parsing is succesfull the xml file is generated
        {
            iter_t iter = result.trees.begin();
            xmlFile = generateModuleLibraryXML(iter);
        }
        else //Failed parsing
        {
            std::map<unsigned long, std::string> fileLines = fileToLines(fileName);
            unsigned long lineNumber = findLine(result.stop,fileLines);

            cerr << "Parsing of file " << fileName;
            cerr << " according to BNF failed in line: " << lineNumber << std::endl;
            //std::cout << result.stop << std::endl;
            exit(EXIT_FAILURE);
        }

        //Create the P system using the xml file
        createFromXML(xmlFile);
    }
    else if(fileExtension == "xml")
    {
        createFromXML(fileName);
    }
    else
    {
        cerr << "Error: unknown file extension in file: " << fileName << std::endl;
        cerr << "Formats supported:\n" ;
        cerr << "\t PLB: text file must be specified according to our BNF \n";
        cerr << "\t      and the file extension must be .plb";
        exit(EXIT_FAILURE);
    }
}


void ModuleLibrary::createFromXML(std::string fileName)
{
    //Check wether the file exists
    FILE* file = fopen(fileName.c_str(), "r");

    if(!file)
    {
      cerr << "Problems loading library file: " << fileName << std::endl;
      exit(EXIT_FAILURE);
    }


    //We use the DOM parser which parses the whole document and stores the
    //structure in memory.
    DomParser parser;

    //We parse our XML file.
    parser.parse_file(fileName);

    if(!parser)
    {
        cerr << "Unsuccesfull parsing of file " << fileName << std::endl;
        exit(EXIT_FAILURE);
    }

    //We get the root node of the Module Library
    xmlpp::Node* rootNode = parser.get_document()->get_root_node();

    xmlpp::Node::NodeList children = rootNode->get_children("moduleDefinition");

    for(xmlpp::Node::NodeList::iterator it = children.begin();
        it != children.end();
        ++it)
    {
        ModuleDefinition mod(*it, this->modules);
        this->modules.insert(pair<string,ModuleDefinition>(mod.getName(),mod));
    }

}

//Displayer
void ModuleLibrary::show()
{

  cout << "================================================" << endl;
  cout << " Module Library: " << name << endl;

  for(map<string,ModuleDefinition>::iterator it = modules.begin();
      it != modules.end();
      ++it)
    {
      (it->second).show();
    }

  cout << "================================================" << endl;

}


