/*
 * File:   ModuleLibraryBNF.cpp
 * Author: Francisco J. Romero-Campero
 *
 * Created on 19 January 2010, 11:05
 */

#include "ModuleLibraryBNF.h"
#include "PSystemBNF.h"

using namespace std;
using namespace xmlpp;
using namespace ECSB;

std::string ECSB::generateModuleLibraryXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* libraryNode = xmlFile.create_root_node("libraryOfModules");

    //Associated p system name
    std::string libraryName(tree->value.begin(), tree->value.end());
    libraryName = removeSpaces(libraryName);
    libraryNode->set_attribute("name",libraryName);

    for(iter_t child = tree->children.begin();
        child != tree->children.end();
        ++child)
    {
        std::string moduleName(child->value.begin(),child->value.end());
        moduleName = removeSpaces(moduleName);
        xmlpp::Element* moduleNode = libraryNode->add_child("moduleDefinition");
        moduleNode->set_attribute("name",moduleName);

        //The different attributes of a module (objects,constants and
        //compartments) and the set of rules associated with it.
        iter_t objectsDef = child->children.begin();
        iter_t constantsDef = child->children.begin() + 1;
        iter_t compartmentsDef = child->children.begin() + 2;
        iter_t ruleSetDef = child->children.begin() + 3;

        //Creating the corresponding xml nodes for the attributes and set of
        //rules.
        xmlpp::Element* objectsNode = moduleNode->add_child("setOfObjects");
        xmlpp::Element* constantsNode = moduleNode->add_child("setOfConstants");
        xmlpp::Element* compartmentsNode = moduleNode->add_child("setOfLabels");
        xmlpp::Element* rulesNode = moduleNode->add_child("setOfRules");

        //Objects
        for(iter_t obj = objectsDef->children.begin();
            obj != objectsDef->children.end();
            ++obj)
        {
            std::string objectName(obj->value.begin(),obj->value.end());
            objectName = removeSpaces(objectName);
            xmlpp::Element* objectNode = objectsNode->add_child("object");
            objectNode->set_attribute("name",objectName);
        }

        //Constants
        for(iter_t cons = constantsDef->children.begin();
            cons != constantsDef->children.end();
            ++cons)
        {
            std::string constantName(cons->value.begin(),cons->value.end());
            constantName = removeSpaces(constantName);
            xmlpp::Element* constantNode = constantsNode->add_child("constant");
            constantNode->set_attribute("name",constantName);
            if(cons->children.size() > 0)
            {
                iter_t bounds = cons->children.begin();

                std::string scale(bounds->value.begin(),bounds->value.end());
                constantNode->set_attribute("scale",scale);

                iter_t lbSpec = bounds->children.begin();
                std::string lowerBound(lbSpec->value.begin(),lbSpec->value.end());
                constantNode->set_attribute("lowerBound",lowerBound);

                iter_t prSpec = bounds->children.begin() + 1;
                std::string precision(prSpec->value.begin(),prSpec->value.end());
                constantNode->set_attribute("precision",precision);

                iter_t ubSpec = bounds->children.begin() + 2;
                std::string upperBound(ubSpec->value.begin(),ubSpec->value.end());
                constantNode->set_attribute("upperBound",upperBound);
            }
        }

        //Compartments
        for(iter_t comp = compartmentsDef->children.begin();
            comp != compartmentsDef->children.end();
            ++comp)
        {
            std::string compartmentName(comp->value.begin(),comp->value.end());
            compartmentName = removeSpaces(compartmentName);
            xmlpp::Element* compartmentNode =
                        compartmentsNode->add_child("label");
            compartmentNode->set_attribute("name",compartmentName);
        }

        //Rules and modules
        for(iter_t r = ruleSetDef->children.begin();
            r != ruleSetDef->children.end();
            ++r)
        {
            int numberChildren = r->children.size();

            std::string ruleName(r->value.begin(),r->value.end());
            ruleName = removeSpaces(ruleName);

            if(numberChildren == 4)
            {
                addModule(r,rulesNode);
            }
            else if(numberChildren == 7 || numberChildren == 6)
            {
                addRule(r,rulesNode);
            }
            else if(numberChildren == 9 || numberChildren == 8)
            {
                addTranslRule(r,rulesNode);
            }
        }
    }

    //Writing the XML file with a formatted style
    std::string fileName = libraryName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;
}
