/* 
 * File:   ModuleLibraryBNF.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 19 January 2010, 11:05
 */

#ifndef _MODULELIBRARYBNF_H
#define	_MODULELIBRARYBNF_H

#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_ast.hpp>
#include <libxml++/libxml++.h>
#include <string>
#include "Misc.h"

using namespace boost::spirit::classic;
using namespace xmlpp;
using namespace std;

namespace ECSB {

////////////////////////////////////////////////////////////////////////////
//  Our Library BNF grammar
////////////////////////////////////////////////////////////////////////////
struct ModuleLibrary_p: public grammar<ModuleLibrary_p>
{
    template <typename ScannerT>
    struct definition
    {
        definition(ModuleLibrary_p const& /*self*/)
        {

            Library = discard_node_d[!(comment)] >> discard_node_d[str_p("libraryOfModules")] >> root_node_d[libraryName] >>
                        +moduleDefinition >>
                      discard_node_d[str_p("endLibraryOfModules")];

            libraryName = leaf_node_d[(lexeme_d[(+(alnum_p))])];//     leaf_node_d[ lexeme_d[+(alnum_p | ch_p('_'))] ];

            moduleDefinition =  discard_node_d[!(comment)] >>
                                root_node_d[moduleName] >> discard_node_d[ str_p("(") ] >>
                                    moduleObjects >>
                                    discard_node_d[ str_p(",") ] >>
                                    moduleConstants >>
                                    discard_node_d[ str_p(",") ] >>
                                    moduleCompartments >>
                                discard_node_d[ str_p(")") ] >>
                    discard_node_d[ str_p("=") ] >>
                    discard_node_d[str_p("{")] >> 
                    !(discard_node_d[!(comment)] >> discard_node_d[type]) >>
                    !(discard_node_d[!(comment)] >> discard_node_d[sequence]) >>
                    discard_node_d[!(comment)] >> ruleSet >>
                    discard_node_d[str_p("}")];

            moduleName = leaf_node_d[ lexeme_d[(+(alnum_p | ch_p('_')))] ];

            moduleObjects = root_node_d[ str_p("{") ] >>
                                !(objectName >> *((discard_node_d[ch_p(',')] >> objectName))) >>
                            discard_node_d[ str_p("}") ];

            moduleConstants = root_node_d[ str_p("{") ] >>
                                !(moduleConstant >> *(discard_node_d[ch_p(',')] >> moduleConstant)) >>
                              //  !(constantName >> *(discard_node_d[ch_p(',')] >> constantName)) >>
                              discard_node_d[ str_p("}") ];

            moduleCompartments = root_node_d[ str_p("{") ] >>
                                    !(labelName >> *((discard_node_d[ch_p(',')] >> labelName))) >>
                                 discard_node_d[ str_p("}") ];

            moduleConstant = root_node_d[constantName] >> !(constantBounds);

            constantBounds = real_p >> discard_node_d[str_p(":")] >> real_p >> discard_node_d[str_p(":")] >> real_p >> root_node_d[scale];



//            constantBounds = root_node_d[constantName] >>
//                             real_p >> discard_node_d[str_p(":")] >> real_p >> discard_node_d[str_p(":")] >> real_p >>
//                             scale;

            scale = str_p("linear") | str_p("logarithmic");
            
            constantName = leaf_node_d[(lexeme_d[(+(alnum_p | ch_p('.') | ch_p('_')))])];

            type = discard_node_d[str_p("type")] >> discard_node_d[ch_p(':')] >> discard_node_d[supportedTypes];

            supportedTypes = discard_node_d[(str_p("promoter") | str_p("RBS") | str_p("riboswitch") | str_p("gene") | str_p("degradation_tag"))];

            sequence = discard_node_d[str_p("sequence")] >> discard_node_d[ch_p(':')] >> discard_node_d[+(nucleotide)];

            nucleotide = discard_node_d[(ch_p('A') | ch_p('T') | ch_p('G') | ch_p('C'))];

            ruleSet = //root_node_d[str_p("{")] >>
                      root_node_d[str_p("rules")] >> discard_node_d[ch_p(':')] >>
                        *(PSRule | module);// >>
                      //discard_node_d[str_p("}")];

            PSRule = BoundaryRule | TranslRule;

            BoundaryRule = discard_node_d[!(comment)] >> root_node_d[ruleName] >> ':' >>
                     LHS  >>
                     ch_p('-') >> constantName >> leaf_node_d[ lexeme_d[str_p("->")]] >>
                     RHS >>
                     !ruleConstant;

            ruleName = leaf_node_d[ (lexeme_d[(+alnum_p)] - (str_p("endRuleSet") | str_p("module"))) ];

            LHS =  LHS_outside >>  LHS_inside >> labelName;
            LHS_outside = multiset >> root_node_d[ch_p('[')];
            LHS_inside = multiset >> root_node_d[str_p("]_")];

            RHS = RHS_outside >> RHS_inside >> labelName;
            RHS_outside = multiset >> root_node_d[ch_p('[')];
            RHS_inside = multiset >> root_node_d[str_p("]_")];

            multiset = !(objectName >> *(root_node_d[ch_p('+')] >> objectName));

            labelName =  leaf_node_d[ lexeme_d[(+alnum_p)] ];

            ruleConstant = constantName >> root_node_d[str_p("=")] >> real_p;

            objectName = leaf_node_d[ (lexeme_d[(+(alnum_p | ch_p('_')))])];

            TranslRule = discard_node_d[!(comment)] >> root_node_d[ruleName] >> ch_p(':') >>
                         LHSTransl  >> ch_p('[') >> ch_p(']') >>
                         ch_p('-') >> constantName >> str_p("->") >>
                         RHSTransl >>
                         !ruleConstant;

            LHSTransl = LHS_objects >> labelName >> ruleVector;

            LHS_objects = root_node_d[ch_p('[')] >> multiset >> discard_node_d[ str_p("]_")];
            ruleVector = discard_node_d[str_p("=")] >> discard_node_d[str_p("(")] >>
                    int_p >> root_node_d[str_p(",")] >> int_p >>
                    discard_node_d[str_p(")")] >> discard_node_d[str_p("=")];

            RHSTransl =  discard_node_d[str_p("[")] >> discard_node_d[str_p("]_")] >>
                         labelName >>  ruleVector >> RHS_objects ;
            RHS_objects = root_node_d[ch_p('[')] >> multiset >> discard_node_d[ str_p("]")];

            module = root_node_d[moduleName] >> discard_node_d[ str_p("(") ] >>
                        moduleObjects >>
                        discard_node_d[ str_p(",") ] >>
                        moduleConstants >>
                        discard_node_d[ str_p(",") ] >>
                        moduleCompartments >>
                    discard_node_d[ str_p(")") ] >>
                    discard_node_d[str_p("from")] >> libraryName;

            libraryName = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('.'))] ];

            commentLine = discard_node_d[ch_p('#')] >> discard_node_d[+(alnum_p | ch_p('_') | ch_p('+') | ch_p('.') | ch_p(':') | ch_p(',') | ch_p('-') | ch_p('*') | ch_p('/') | ch_p('(') | ch_p(')') | ch_p('='))] >> discard_node_d[ch_p('#')];

            comment = +(commentLine);
        }

        rule<ScannerT> Library, libraryName, moduleDefinition, moduleName, moduleObjects, moduleConstants, moduleCompartments, moduleConstant, constantBounds, scale, ruleSet, PSRule, BoundaryRule, ruleName, LHS, LHS_outside, LHS_inside, RHS, RHS_outside, RHS_inside, labelName, multiset, constantName, ruleConstant, objectName, TranslRule, LHSTransl, LHS_objects, ruleVector, RHSTransl, RHS_objects, module, type, supportedTypes, sequence, nucleotide, commentLine, comment, constantNoBounds;

        rule<ScannerT> const&
        start() const { return Library; }
    };
};

std::string generateModuleLibraryXML(iter_t tree);

}

#endif	/* _MODULELIBRARYBNF_H */

