/*
 * File:   psystem.h
 * Author: fran
 *
 * Created on 15 January 2010, 16:31
 */

#ifndef _PSYSTEM_H
#define	_PSYSTEM_H

#include "PSystemBNF.h"
#include "LPPCompartment.h"
#include "InitialMultiset.h"
#include "LPPRuleSet.h"
#include <sbml/SBMLTypes.h>

using namespace std;
using namespace xmlpp;

namespace ECSB
{

class PSystem {

public:
    //Constructors
    PSystem();
    PSystem(const PSystem& orig);
    PSystem(std::string fileName);

    //Default destructor
    ~PSystem();

    //Getters
    std::string getName() { return name; }
    std::vector<std::string> getAlphabet() { return alphabet; }
    std::vector<std::string> getSetOfLabels() { return setOfLabels; }
    std::map<std::string,LPPCompartment> getMembraneStructure()
    {
        return membraneStructure;
    }
    LPPCompartment getCompartment(std::string compName)
    {
        return membraneStructure[compName];
    }
    std::map<std::string,InitialMultiset> getInitialMultisets()
    {
        return initialMultisets;
    }
    InitialMultiset getInitialMultiset(std::string iniName)
    {
        return initialMultisets[iniName];
    }
    std::map<std::string,LPPRuleSet> getRuleSets()
    {
        return ruleSets;
    }
    LPPRuleSet getRuleSet(std::string ruleSetName)
    {
        return ruleSets[ruleSetName];
    }
    std::list<std::string> getRuleObjects();
    unsigned long getNumberOfCompartments()
    {
        return (this->membraneStructure).size();
    }

    //Setters
    void setName(std::string n) { name = n; }
    void setAlphabet(std::vector<std::string> a) { alphabet = a; }
    void setSetOfLabels(std::vector<std::string> s) { setOfLabels = s; }
    void setMembraneStructure(std::map<std::string,LPPCompartment> m)
    {
        membraneStructure = m;
    }
    void addCompartment(LPPCompartment comp)
    {
        membraneStructure[comp.getName()] = comp;
    }
    void setInitialMultisets(std::map<std::string,InitialMultiset> i)
    {
        initialMultisets = i;
    }
    void addInitialMultiset(InitialMultiset ini)
    {
        initialMultisets[ini.getLabel()] = ini;
    }
    void setObjectMultiplicity(std::string label,
                               std::string objectName,
                               unsigned long objectMultiplicity)
    {
        (this->initialMultisets)[label].setMultiplicity(objectName,
                                                        objectMultiplicity);
    }
    void setRuleSets(std::map<std::string,LPPRuleSet> r)
    {
        ruleSets = r;
    }
    void addRuleSet(LPPRuleSet rs)
    {
        ruleSets[rs.getLabel()] = rs;
    }
    void addRule(std::string compartmentName, LPPRule rule);

    //Converter to translocation rules
    void convertTranslocation();

    //Displayer
    void show();


protected:
    //Auxiliary functions to create a P system from an XML file
    void createFromXML(std::string fileName);
    void createAlphabet(xmlpp::Node::NodeList alphabetDeclaration);
    void createSetOfLabels(xmlpp::Node::NodeList labelsDeclaration);
    void createMembraneStructure(xmlpp::Node::NodeList compartmentsDeclaration);
    std::vector<std::string> findChildren(std::string compartmentName);
    void assignChildren();
    void createInitialMultisets(xmlpp::Node::NodeList initialMultisetsDeclaration);
    void createRuleSets(xmlpp::Node::NodeList ruleSetsDeclaration);
    void checkAlphabet();

    //Auxiliary functions to create a P system from an SBML file
    void createFromSBML(std::string fileName);
    void createAlphabet(Model* sbmlModel);
    bool isDegradation(std::string s);
    void createMembraneStructure(Model* sbmlModel);
    void createSetOfLabels();
    void createInitialMultisets(Model* sbmlModel);
    void createRuleSets(Model* sbmlModel);
    void checkValid(Model* sbmlModel, Reaction* reaction);
    std::string reactionType(Model* sbmlModel, Reaction* reaction);
    std::vector<std::string> reactants(Model* sbmlModel, Reaction* reaction);
    std::vector<std::string> products(Model* sbmlModel, Reaction* reaction);

    //Auxiliary function that converts rules that move objects from the ouside
    //to the inside of comparments and viceversa in the hierarchical structure of
    //membranes
    LPPRule convertTranslocationRule(LPPRule rule, std::string compartmentName);


private:
    std::string name;
    std::vector<std::string> alphabet;
    std::vector<std::string> setOfLabels;
    std::map<std::string,LPPCompartment> membraneStructure;
    std::map<std::string,InitialMultiset> initialMultisets;
    std::map<std::string,LPPRuleSet> ruleSets;

};

}

#endif	/* _PSYSTEM_H */

