/* 
 * File:   PSystemBNF.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 17 January 2010, 20:26
 */

#ifndef _PSYSTEMBNF_H
#define	_PSYSTEMBNF_H

#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_ast.hpp>
#include <libxml++/libxml++.h>
#include <string>
#include "Misc.h"

using namespace std;
using namespace xmlpp;
using namespace boost::spirit::classic;

namespace ECSB
{

////////////////////////////////////////////////////////////////////////////
//  Our P system BNF grammar
////////////////////////////////////////////////////////////////////////////
struct Psystem_p: public grammar<Psystem_p>
{
    template <typename ScannerT>
    struct definition
    {
        definition(Psystem_p const& /*self*/)
        {

            Psystem = discard_node_d[!(comment)] >>
                      discard_node_d[str_p("SPsystem")] >> root_node_d[PsystemName] >>
                        discard_node_d[!(comment)] >>
                        alphabet >>
                        discard_node_d[!(comment)] >>
                        compartments >>
                        discard_node_d[!(comment)] >>
                        initialMultisets >>
                        discard_node_d[!(comment)] >>
                        ruleSets >>
                      discard_node_d[str_p("endSPsystem")];

            PsystemName = leaf_node_d[ lexeme_d[(+(alnum_p | ch_p('_')))] ];

            alphabet = root_node_d[str_p("alphabet")] >>
                            *(discard_node_d[!(comment)] >> objectName)>>
                            //+(objectName)>>
                       discard_node_d[ str_p("endAlphabet") ];

            objectName = leaf_node_d[ (lexeme_d[(+(alnum_p | ch_p('_')))] - (str_p("endAlphabet") | str_p("endInitialMultiset")))];

            compartments = root_node_d[str_p("compartments")] >>
                                +(discard_node_d[!(comment)] >> compartment) >>
                                //+(compartment) >>
                           discard_node_d[str_p("endCompartments")];

            compartment = root_node_d[compartmentName] >> !(discard_node_d[ str_p("inside") ] >> compartmentName);

            compartmentName = leaf_node_d[ (lexeme_d[ ((+(alnum_p | ch_p('_'))) - (str_p("in") | str_p("endCompartments"))) ])];

            initialMultisets = root_node_d[str_p("initialMultisets")] >>
                                    +(discard_node_d[!(comment)] >> initialMultiset) >>
                                    //+(initialMultiset) >>
                               discard_node_d[str_p("endInitialMultisets")];

            initialMultiset = discard_node_d[str_p("initialMultiset")] >> root_node_d[compartmentName] >>
                                    *(discard_node_d[!(comment)] >> multiplicity) >>
                                    //*(multiplicity) >>
                              discard_node_d[str_p("endInitialMultiset")];

            multiplicity = root_node_d[objectName] >> int_p;

            ruleSets = root_node_d[str_p("ruleSets")] >>
                            +(ruleSet) >>
                       discard_node_d[str_p("endRuleSets")];

            ruleSet = discard_node_d[!(comment)] >> discard_node_d[str_p("ruleSet")] >> root_node_d[compartmentName] >>
                        *(PSRule | module) >>
                      discard_node_d[str_p("endRuleSet")];

            PSRule = BoundaryRule | TranslRule;

            BoundaryRule = discard_node_d[!(comment)] >> root_node_d[ruleName] >> ch_p(':') >>
                           LHS  >>
                           ch_p('-') >> constantName >> leaf_node_d[ lexeme_d[str_p("->")]] >>
                           RHS >>
                           ruleConstant;

            ruleName = leaf_node_d[ (lexeme_d[(+alnum_p)] - (str_p("endRuleSet") | str_p("module"))) ];

            LHS =  LHS_outside >>  LHS_inside >> labelName;
            LHS_outside = multiset >> root_node_d[ch_p('[')];
            LHS_inside = multiset >> root_node_d[str_p("]_")];

            RHS = RHS_outside >> RHS_inside >> labelName;
            RHS_outside = multiset >> root_node_d[ch_p('[')];
            RHS_inside = multiset >> root_node_d[str_p("]_")];

            ruleConstant = constantName >> root_node_d[ch_p('=')] >> real_p;

            multiset = !(objectName >> *(root_node_d[ch_p('+')] >> objectName));

            labelName =  leaf_node_d[ lexeme_d[(+alnum_p)] ];

            constantName = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('.') | ch_p('_'))] ];

            TranslRule = discard_node_d[!(comment)] >> root_node_d[ruleName] >> ch_p(':') >>
                         LHSTransl  >> ch_p('[') >> ch_p(']') >>
                         ch_p('-') >> constantName >> str_p("->") >>
                         RHSTransl >>
                         ruleConstant;

            LHSTransl = LHS_objects >> labelName >> ruleVector;

            LHS_objects = root_node_d[ch_p('[')] >> multiset >> discard_node_d[ str_p("]_")];
            ruleVector = discard_node_d[str_p("=")] >> discard_node_d[str_p("(")] >>
                    int_p >> root_node_d[str_p(",")] >> int_p >>
                    discard_node_d[str_p(")")] >> discard_node_d[str_p("=")];

            RHSTransl =  discard_node_d[str_p("[")] >> discard_node_d[str_p("]_")] >>
                         labelName >>  ruleVector >> RHS_objects ;
            RHS_objects = root_node_d[ch_p('[')] >> multiset >> discard_node_d[ str_p("]")];


            module = discard_node_d[!(comment)] >>
                    root_node_d[moduleName] >> discard_node_d[ str_p("(") ] >>
                        moduleObjects >>
                        discard_node_d[ str_p(",") ] >>
                        moduleConstants >>
                        discard_node_d[ str_p(",") ] >>
                        moduleCompartments >>
                    discard_node_d[ str_p(")") ] >>
                    discard_node_d[str_p("from")] >> libraryName;

            moduleName = leaf_node_d[ lexeme_d[(+(alnum_p | ch_p('_')))] ];

            libraryName = leaf_node_d[ lexeme_d[+(alnum_p | ch_p('_') | ch_p('.'))] ];

            moduleObjects = root_node_d[ str_p("{") ] >>
                                !(objectName >> *((discard_node_d[ch_p(',')] >> objectName))) >>
                            discard_node_d[ str_p("}") ];

            moduleConstants = root_node_d[ str_p("{") ] >>
                                !(constantName >> *(discard_node_d[ch_p(',')] >> constantName)) >>
                              discard_node_d[ str_p("}") ];

            moduleCompartments = root_node_d[ str_p("{") ] >>
                                    !(labelName >> *((discard_node_d[ch_p(',')] >> labelName))) >>
                                 discard_node_d[ str_p("}") ];

            commentLine = discard_node_d[ch_p('#')] >> discard_node_d[+(alnum_p | ch_p('_') | ch_p('+') | ch_p('.') | ch_p(':') | ch_p(',') | ch_p('-') | ch_p('*') | ch_p('/') | ch_p('(') | ch_p(')'))] >> discard_node_d[ch_p('#')];

            comment = +(commentLine);


        }

        rule<ScannerT> Psystem, PsystemName, alphabet, objectName, compartments, compartment, elementary, nonElementary, initialMultisets, initialMultiset, multiplicity, ruleSets, ruleSet, compartmentName, PSRule, BoundaryRule, LHS, LHS_outside, LHS_inside, RHS, RHS_outside, RHS_inside, ruleConstant, TranslRule, LHSTransl, LHS_objects, ruleVector, RHSTransl, RHS_objects, multiset, labelName, constantName, ruleName, module, moduleName, libraryName, moduleObjects, moduleConstants, moduleCompartments, commentLine, comment;

        rule<ScannerT> const&
        start() const { return Psystem; }
    };
};

//Functions used to generate an XML specification using our BNF grammar.
std::string generatePSystemXML(iter_t tree);
void addAlphabet(xmlpp::Element* alphabetNode, iter_t child);
void addCompartmentLabels(xmlpp::Element* compartmentLabelsNode, iter_t compartments);
void addMembraneStructure(xmlpp::Element* compartmentsNode, iter_t compartments);
void addInitialMultisets(xmlpp::Element* initialMultisetsNode, iter_t initialMultisets);
void addRuleSets(xmlpp::Element* ruleSetsNode, iter_t ruleSets);
void addModule(iter_t ruleMod, xmlpp::Element* ruleSetNode);
void addRule(iter_t ruleMod, xmlpp::Element* ruleSetNode);
void addTranslRule(iter_t ruleMod, xmlpp::Element* ruleSetNode);
unsigned long reportErrorLine(std::string errorString, std::string fileName);

}

#endif	/* _PSYSTEMBNF_H */

