// parameter set class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "ParameterSet.h"

#include <iostream>
#include <string>
#include <libxml++/libxml++.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

using namespace ECSB;
using namespace std;
using namespace xmlpp;

// constructor
ParameterSet::ParameterSet(const char *name)
{
	classname = name;
}

// return name of class
string ParameterSet::getClassName(void)
{
	return classname;
}

// return filename of parameter set
string ParameterSet::getFilename(void)
{
	return filename;
}

// read parameters from an xml file
void ParameterSet::read(char *filename)
{
	Node *node;
	Node::NodeList nodeList;
	Node::NodeList::iterator iter;
	string pVal, pName;
	
	// load xml document
	try {
		DomParser parser(filename);
		ParameterSet::filename = filename;
		Document *document = parser.get_document();
	
		// check root node
		if(document->get_root_node()->get_name() != "parameters") {
			cerr << "error: no parameters element\n";
			exit(EXIT_FAILURE);
		}	

		// get list of parameter sets
		nodeList = document->get_root_node()->get_children("parameterSet");
		if(nodeList.empty()) {
			cerr << "error: no parameterSet elements specified\n";
			exit(EXIT_FAILURE);
		}

		// find parameter set corresponding to this class
		for(iter = nodeList.begin(); iter != nodeList.end(); ++iter) {
			pVal = (dynamic_cast<Element *>(*iter))-> \
					get_attribute_value("name");
			if(pVal == getClassName())
				break;
		}	
		if(pVal != getClassName()) {
			cerr << "error: no parameterSet element for " << getClassName() \
					<< " found\n";
			exit(EXIT_FAILURE);
		}
	
		// get list of parameters
		nodeList = (*iter)->get_children("parameter");
		if(nodeList.empty()) {
			cerr << "error: no parameter elements for " << getClassName() \
					<< " found\n";
			exit(EXIT_FAILURE);
		}

		// parse each parameter
		for(iter = nodeList.begin(); iter != nodeList.end(); ++iter) {
			node = *iter;
			// get parameter name
			pName = (dynamic_cast<Element *>(node))-> \
					get_attribute_value("name");
			if(pName.empty()) {
				cerr << "error: no name attribute specified for parameter\n";
				exit(EXIT_FAILURE);
			}

			// get parameter type
			pVal = (dynamic_cast<Element *>(node))-> \
					get_attribute_value("value");
			if(pVal.empty()) {
				cerr << "error: no value attribute specified for parameter " \
						<< pName << "\n";
				exit(EXIT_FAILURE);
			}
			if(!setParameter((char *) pName.c_str(), (char *) pVal.c_str())) {
				cerr << "error: unknown parameter " << pName << "\n";
				exit(EXIT_FAILURE);
			}
		}
	}
	catch(const std::exception &ex) {
		cerr << "error: couldn't read parameter file " << filename << "\n";
		exit(EXIT_FAILURE);
	}
}

// read parameters from an xml file and command line
void ParameterSet::read(int argc, char **argv)
{
	int ind;
	char pName[1024], *pVal;

	// check command line
	if(argc < 2) {
		cerr << "usage: " << argv[0] << " PARAMETER_FILE [PARAMETER=VALUE] ...\n";
		exit(EXIT_FAILURE);
	}
	
	// read parameters from file
	read(argv[1]);
	
	// read parameters specified on command line
	ind = 2;
	while(ind < argc) {
		strncpy(pName, argv[ind], 1024);
		if(!(pVal = strchr(pName, '='))) {
			cerr << "error: couldn't parse command line parameter " << \
					argv[ind] << "\n";
			exit(EXIT_FAILURE);
		}
		*pVal++ = '\0';
		if(!setParameter(pName, pVal)) {
			cerr << "error: unknown parameter " << pName << "\n";
			exit(EXIT_FAILURE);
		}
 		ind++;
	}
}
