// random number generation functions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "RandomNumber.h"

#include <ctime>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

using namespace ECSB;

static gsl_rng *rng;  // random number generator

// create a random number generator
unsigned long ECSB::initialiseRandomNumberGenerator(unsigned long seed = 0)
{
	FILE *fp;
	unsigned long newseed;

	// initialise uniform random number generator
	rng = gsl_rng_alloc(gsl_rng_mt19937);
	
	// seed generator randomly if seed set to zero
	newseed = seed;
	if(!newseed) {
		if((fp = fopen("/dev/urandom", "r"))) {
			// set seed from /dev/urandom
			fread(&newseed, sizeof(newseed), 1, fp);
			fclose(fp);
		} else
			// set seed from time
			newseed = (unsigned long) std::time(0);
	}
	gsl_rng_set(rng, newseed);

	return newseed;
}

// release memory for random number generator
void ECSB::freeRandomNumberGenerator(void)
{
	gsl_rng_free(rng);
}

// generate a uniform random number on the interval [0,1)
double ECSB::randomNumber(void)
{
	return gsl_rng_uniform(rng);
}

// generate a uniform random number on the interval (0,1)
double ECSB::randomNumberPos(void)
{
	return gsl_rng_uniform_pos(rng);
}

// generate a uniform random non-negative integer
unsigned long ECSB::randomNonnegativeInteger(unsigned long range)
{
	return gsl_rng_uniform_int(rng, range);
}


// generate a gaussian random number
double ECSB::gaussianRandomNumber(double mean, double sd)
{
	return (mean + gsl_ran_gaussian(rng, sd));
}

// generate a non-negative gaussian random number
double ECSB::gaussianRandomNumberNN(double mean, double sd)
{
	register double r;

	r = mean + gsl_ran_gaussian(rng, sd);
	return ((r < 0.0) ? 0.0 : r );
}

// divide integer by two and round up or down randomly
unsigned long ECSB::randomHalf(unsigned long val)
{
	if(!(val % 2))
		return (val / 2);
	return ((val / 2) + gsl_rng_uniform_int(rng, 2));
}
