// signal handler functions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "SignalHandler.h"
#include <iostream>
#include <stdlib.h>

using namespace ECSB;
using namespace std;

Slot SignalHandler::handler[MAX_SIGNAL];
bool SignalHandler::handler_ignore[MAX_SIGNAL];

// set handler for signal
void SignalHandler::set_handler(unsigned int sig, Slot slot)
{
	// sanity check
	if(sig >= MAX_SIGNAL || slot.empty()) {
		cerr << "error: couldn't set signal handler\n";
		exit(EXIT_FAILURE);
	}

	// set handler
	signal(sig, &SignalHandler::caught);
	handler[sig] = slot;
	handler_ignore[sig] = false;
}

// unset handler for signal
void SignalHandler::unset_handler(unsigned int sig)
{
	// sanity check
	if(sig >= MAX_SIGNAL) {
		cerr << "error: couldn't unset signal handler\n";
		exit(EXIT_FAILURE);
	}

	// reset handler
	signal(sig, SIG_DFL);
	handler_ignore[sig] = false;
}

// execute handler for signal
void SignalHandler::caught(int sig)
{
	// sanity check
	if(sig >= MAX_SIGNAL) {
		cerr << "error: received invalid signal\n";
		exit(EXIT_FAILURE);
	}

	// execute handler if one exists
	if(!handler[sig].empty() && !handler_ignore[sig])
		handler[sig]();
}

// ignore previously set signal
void SignalHandler::ignore_handler(unsigned int sig)
{
	// sanity check
	if(sig >= MAX_SIGNAL) {
		cerr << "error: couldn't ignore signal handler\n";
		exit(EXIT_FAILURE);
	}

	// ignore signal
	handler_ignore[sig] = true;
}

// unignore previously set signal
void SignalHandler::unignore_handler(unsigned int sig)
{
	// sanity check
	if(sig >= MAX_SIGNAL) {
		cerr << "error: couldn't unignore signal handler\n";
		exit(EXIT_FAILURE);
	}

	// unignore signal
	handler_ignore[sig] = false;
}
