/*
 * File:   spatialdistribution.h
 * Author: Francisco J. Romero-Campero
 *
 * Created on 15 January 2010, 09:43
 */

#ifndef _SPATIALDISTRIBUTION_H
#define	_SPATIALDISTRIBUTION_H

#include <muParser.h>

#include "Parameter.h"
#include "PSystem.h"

using namespace std;
using namespace xmlpp;

namespace ECSB
{

class spatialDistribution {

public:

    //Constructors
    spatialDistribution();
    spatialDistribution(const spatialDistribution& orig);
    spatialDistribution(xmlpp::Node* Declaration,
                        std::map<std::string,PSystem> pSystems);

    //Default destructor
    ~spatialDistribution();

    //Getters
    std::string getName() { return name; }
    std::map<std::vector<long>, std::string>  getPositions()
    {
        return positions;
    }

    std::map<std::vector<long>, std::string>*  getPositionsPtr()
    {
        return &positions;
    }

    unsigned long getNumberOfCompartments()
    {
        return numberOfCompartments;
    }
    std::string getDistribution()
    {
        return distribution;
    }
    double getNoise()
    {
        return noise;
    }
    int getNumberMutations()
    {
        return numberMutations;
    }

    //Setters
    void setName(std::string n)
    {
        name = n;
    }
    void setPositions(std::map<std::vector<long>, std::string> pos)
    {
        positions = pos;
    }
    //Adding a position and P system overwritting the P system previously
    //associated with the positions if any.
    void addPositionPsystem(std::vector<long> pos, PSystem pSystem)
    {
        positions[pos] = pSystem.getName();
    }
    //Adds a positions and a P system if the position was not associated any
    //P system previously otherwise it sends an error message
    void insertPositionPsystem(std::vector<long> pos, PSystem pSystem)
    {
        if(positions.find(pos) == positions.end())
            positions[pos] = pSystem.getName();
        else
        {
            cerr << "Position (" << pos[0] << "," << pos[1] << ") has already ";
            cerr << "the P system " << positions[pos] << std::endl;
            exit(EXIT_FAILURE);
        }
    }
    void setNumberOfCompartments(unsigned long comp)
    {
        numberOfCompartments = comp;
    }
    void setDistribution(std::string dist)
    {
        distribution = dist;
    }
    void setNoise(double n)
    {
        noise = n;
    }
    void setNumberMutations(int mut)
    {
        numberMutations = mut;
    }

    //Displayer
    void show();

    //Fucntion adding noise to the initial spatial distribution
    void addNoise(double probability, long xLowerValue, long xUpperValue,
                  long yLowerValue, long yUpperValue);

protected:

    //Auxiliary function to create fixed positions
    void fixedPositions(xmlpp::Node::NodeList positionsDeclaration,
                        std::map<std::string,PSystem> pSystems);

    //Auxiliary function to create random positions
    void randomPositionAutomata(xmlpp::Node::NodeList automataDeclaration, std::map<std::string,PSystem> pSystems);


    //Auxiliary function to expand the coordinates
    std::vector< std::vector<long> > expandCoordinates(xmlpp::Node* pos);

    //Auxiliary function to get the paramters associated with coordinates
    std::vector<ECSB::parameter> coordinateParameters(xmlpp::Node* pos);

    //Auxiliary functions to combine coordinates
    std::vector< std::vector<long> > combineParametricCoordinates(std::vector< std::vector<long> > coordinatesPrecursor);
    std::vector< std::vector<long> > combineCoordinates(std::vector< std::vector<long> > coordinatesPrecursor);

    //Auxiliary function to evaluate a parametric expression
    std::vector<long> evaluateExpression(std::string expression, std::vector<ECSB::parameter> parameters);

    //Auxiliary function to combine the values of parameters
    std::vector< std::vector<double> > combineValues(std::vector< std::vector<double> > separateValues);

    //Auxiliary function to combine the coordinates when creating random
    //coordinates
    std::vector< std::vector <std::vector<int> > > combineRandomCoordinates(std::vector< std::vector<int> >, int);

    //Auxiliary function to obtain a random neighboruing position
    std::vector<long> neighbouringPosition(std::vector<long> pos);


private:

    std::string name;
    std::map<std::vector<long>, std::string>  positions;
    unsigned long numberOfCompartments;
    std::string distribution;
    double noise;
    int numberMutations;

};

}

#endif	/* _SPATIALDISTRIBUTION_H */

