#!/bin/sh

PKGBASE=$(cat NAME)
VERSION=$(cat VERSION)
PKGNAME=$PKGBASE-$VERSION
CTRLFILE=control
LICENSE="/usr/share/doc/$PKGNAME/COPYING"
README="/usr/share/doc/$PKGNAME/README"
MAINTAINER=$(cat $CTRLFILE | egrep '^%vendor' | sed 's/%vendor //')
LASTVERSION=$(echo $VERSION | cut -f 3 -d '.')
LASTVERSION=$(echo "${LASTVERSION}-1" | bc)
LASTVERSION="$(echo $VERSION | cut -f1,2 -d '.').${LASTVERSION}"

echo "creating ${PKGBASE} debian packages..."

echo "creating debian package..."
CWD=$(pwd)
TMPDIR=/tmp/$PKGNAME
TMPFILE=${TMPDIR}/${PKGNAME}.tmp
LOGFILE=${TMPDIR}/${PKGNAME}.log
ARCH=$(dpkg --print-architecture)

echo "building source..."
rm -rf $TMPDIR &&
mkdir $TMPDIR &&
./autogen.sh &>> ${LOGFILE} &&
./configure --prefix=$TMPDIR/build &>> ${LOGFILE} &&
make clean &>> ${LOGFILE} &&
make all install &>> ${LOGFILE}
if [ $? != 0 ]
then
	echo "error: couldn't build source"
	exit 1
fi

# fix paths
STRING=$(echo "$TMPDIR/build" | sed 's/\//\\\//g') &&
for i in lib/pkgconfig/$PKGBASE.pc lib/$PKGBASE.la
do
	cat $TMPDIR/build/$i | sed "s/$STRING/\/usr/g" > $TMPFILE &&
	mv $TMPFILE $TMPDIR/build/$i
	if [ $? != 0 ]
	then
		echo "error: couldn't fix paths"
		exit 1
	fi
done
if [ $? != 0 ]
then
	exit 1
fi

# create control file
echo "creating binary debian package..."
# create control file header
cp $CTRLFILE $TMPDIR/$PKGNAME.list &&
cat << EOF >> $TMPDIR/$PKGNAME.list &&
%product $PKGNAME
%copyright $LICENSE
%version $VERSION
%readme $README
%replaces ${PKGBASE}-${LASTVERSION}
EOF

# create filelist
mkepmlist -g root -u root --prefix /usr $TMPDIR/build >> $TMPDIR/$PKGNAME.list &&
cd $TMPDIR &&
epm -a $ARCH --output-dir . -v -f deb -n $PKGNAME &>> ${LOGFILE} &&
mv ${PKGNAME}-$VERSION.deb $CWD/${PKGNAME}_${VERSION}_$ARCH.deb &&
cd $CWD
if [ $? != 0 ]
then
	echo "error: couldn't create binary package"
	exit 1
fi

# make source package
echo "creating source package..."
make dist &>> ${LOGFILE} &&
cd $TMPDIR &&
tar -xzvf $CWD/$PKGBASE-$VERSION.tar.gz &> /dev/null &&
mv $PKGBASE-$VERSION $PKGBASE-$VERSION-$VERSION &&
cd $PKGBASE-$VERSION-$VERSION &&
dpkg -e $CWD/${PKGNAME}_${VERSION}_$ARCH.deb &> /dev/null &&
mv DEBIAN debian &&
cat debian/control | egrep '(^Package)|(^Maintainer)|(^Description)' > debian/1 &&
mv debian/1 debian/control &&
cat << EOF >> debian/control &&
Source: $PKGNAME
Architecture: any
Section: science
Priority: optional
EOF
cat << EOF > debian/changelog &&
$PKGNAME ($VERSION) unstable; urgency=low
  * comment here.

 -- $MAINTAINER  $(date '+%a, %d %b %Y %T %z')
EOF
cd $TMPDIR &&
dpkg-source -b $PKGBASE-$VERSION-$VERSION &>> ${LOGFILE} &&
mv $PKGBASE-${VERSION}_$VERSION.tar.gz $PKGBASE-${VERSION}_$VERSION.dsc $CWD/ &&
cd $CWD
if [ $? != 0 ]
then
	echo "error: couldn't create source package"
	exit 1
fi

# make meta package
echo "creating meta package..."
# create control file
mkdir -p $TMPDIR/$PKGBASE/DEBIAN &&
cat << EOF > $TMPDIR/$PKGBASE/DEBIAN/control &&
Package: $PKGBASE
Version: $VERSION
Section: science
Priority: optional
Architecture: all
Depends: $PKGNAME
Replaces: $PKGBASE (<< $VERSION)
EOF
cat $CTRLFILE | egrep '^%vendor' | sed 's/%vendor/Maintainer:/' >> $TMPDIR/$PKGBASE/DEBIAN/control &&
cat $CTRLFILE | egrep '^%description' | sed 's/%description/Description:/' >> $TMPDIR/$PKGBASE/DEBIAN/control &&

# make meta package
cd $TMPDIR &&
dpkg-deb --build $PKGBASE &>> ${LOGFILE} &&
mv $PKGBASE.deb $CWD/${PKGBASE}_${VERSION}_all.deb &&
cd $CWD
if [ $? != 0 ]
then
	echo "error: couldn't creating meta package"
	exit 1
fi

# remove tmp files
rm -rf $TMPDIR

echo "all ${PKGBASE} packages built ok"
